@@ -1,9 +1,10 @@
-/* Copyright (C) 2013-2022 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2013-2024 by Arm Limited. All rights reserved. */
 
 #ifndef CPU_UTILS_H
 #define CPU_UTILS_H
 
 #include "lib/Span.h"
+#include "lib/midr.h"
 
 #include <string>
 
@@ -13,7 +14,7 @@ namespace cpu_utils {
     /**
      * @return hardware name if found or empty
      */
-    std::string readCpuInfo(bool ignoreOffline, bool wantsHardwareName, lib::Span<int> cpuIds);
+    std::string readCpuInfo(bool ignoreOffline, bool wantsHardwareName, lib::Span<midr_t> midrs);
 }
 
 #endif // CPU_UTILS_H
