@@ -1,4 +1,4 @@
-/* Copyright (C) 2014-2023 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2014-2024 by Arm Limited. All rights reserved. */
 
 #include "FtraceDriver.h"
 
@@ -19,14 +19,12 @@
 #include <atomic>
 #include <cerrno>
 #include <chrono>
-#include <csignal>
 #include <cstddef>
 #include <cstdint>
 #include <cstdlib>
 #include <cstring>
 #include <functional>
 #include <memory>
-#include <thread>
 #include <utility>
 #include <vector>
 
@@ -357,7 +355,7 @@ namespace {
 
         {
             // Read any slop
-            std::array<char, 65536> buf {};
+            std::array<char, 65536> buf {}; // NOLINT(readability-magic-numbers)
             ssize_t bytes;
             size_t size;
 
@@ -395,7 +393,6 @@ FtraceDriver::FtraceDriver(const TraceFsConstants & traceFsConstants,
                            size_t numberOfCores)
     : SimpleDriver("Ftrace"),
       traceFsConstants(traceFsConstants),
-      mBarrier(),
       mTracingOn(0),
       mSupported(false),
       mMonotonicRawSupport(false),
@@ -433,6 +430,13 @@ void FtraceDriver::readEvents(mxml_node_t * const xml)
         return;
     }
 
+    // Can the trace events be enabled? (may be restricted by container)
+    if (::access(traceFsConstants.path__events__enable, W_OK) != 0) {
+        mSupported = false;
+        LOG_SETUP("Ftrace is disabled\nUnable to write to \"%s\"", traceFsConstants.path__events__enable);
+        return;
+    }
+
     if (geteuid() != 0) {
         mSupported = false;
         LOG_SETUP("Ftrace is disabled\nFtrace is not supported when running non-root");
