@@ -1,31 +1,27 @@
-/* Copyright (C) 2010-2022 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2010-2024 by Arm Limited. All rights reserved. */
 
 #ifndef SESSION_XML_H
 #define SESSION_XML_H
 
 #include <mxml.h>
 
-struct ImageLinkList;
+namespace {
+    constexpr const size_t BUFFER_SIZE = 64;
+}
 
 struct ConfigParameters {
-    // buffer mode, "streaming", "low", "normal", "high" defines oneshot and buffer size
-    char buffer_mode[64];
-    // capture mode, "high", "normal", or "low"
-    char sample_rate[64];
-    // whether stack unwinding is performed
-    bool call_stack_unwinding;
-    int live_rate;
-
-    ConfigParameters() : buffer_mode(), sample_rate(), call_stack_unwinding(false), live_rate(0)
-    {
-        buffer_mode[0] = '\0';
-        sample_rate[0] = '\0';
-    }
+    /// buffer mode, "streaming", "low", "normal", "high" defines oneshot and buffer size
+    char buffer_mode[BUFFER_SIZE] = {0};
+    /// capture mode, "high", "normal", or "low"
+    char sample_rate[BUFFER_SIZE] = {0};
+    /// whether stack unwinding is performed
+    bool call_stack_unwinding = false;
+    int live_rate = 0;
 };
 
 class SessionXML {
 public:
-    ConfigParameters parameters {};
+    ConfigParameters parameters;
 
     SessionXML(const char * str);
 
