@@ -109,7 +109,7 @@ namespace {
             // add 64 for signal to differentiate from normal exit.
             // can't use 128 to 255 because that would be used by a shell
             // if this process (gator-main) signalled.
-            exitStatus = 64 + signal;
+            exitStatus = 64 + signal; //NOLINT(readability-magic-numbers)
         }
 
         assert(currentStateAndChildPid.state != State::IDLE);
@@ -150,12 +150,12 @@ namespace {
 
     class StreamlineCommandHandler : public IStreamlineCommandHandler {
     public:
-        State handleRequest(char *) override
+        State handleRequest(char * /*xml*/) override
         {
             LOG_DEBUG("INVESTIGATE: Received unknown command type COMMAND_REQUEST_XML");
             return State::PROCESS_COMMANDS;
         }
-        State handleDeliver(char *) override
+        State handleDeliver(char * /*xml*/) override
         {
             LOG_DEBUG("INVESTIGATE: Received unknown command type COMMAND_DELIVER_XML");
             return State::PROCESS_COMMANDS;
@@ -222,13 +222,13 @@ namespace {
                                                              gSessionData.mWaitForProcessCommand,
                                                              gSessionData.mCaptureWorkingDir,
                                                              gSessionData.mPids);
-            sender.writeData(currentConfigXML.data(), currentConfigXML.size(), ResponseType::CURRENT_CONFIG, true);
+            sender.writeData(currentConfigXML, ResponseType::CURRENT_CONFIG, true);
         }
         else if (result != IStreamlineCommandHandler::State::EXIT_DISCONNECT) {
             // the expectation is that the user sends COMMAND_DISCONNECT, so anything else is an error
             LOG_ERROR("Session already in progress");
             std::string last_error = log_ops->get_last_log_error();
-            sender.writeData(last_error.data(), last_error.size(), ResponseType::ERROR, true);
+            sender.writeData(last_error, ResponseType::ERROR, true);
         }
 
         // Ensure all data is flushed the host receive the data (not closing socket too quick)
@@ -349,7 +349,7 @@ namespace {
             ss << errno;
             throw GatorException(ss.str());
         }
-        else if (pid == 0) {
+        if (pid == 0) {
             // Child
             for (const auto & driver : drivers.getAll()) {
                 driver->postChildForkInChild();
@@ -397,12 +397,14 @@ int capture::beginCaptureProcess(const ParserResult & result,
 {
     // Set to high priority
     if (setpriority(PRIO_PROCESS, lib::gettid(), high_priority) == -1) {
-        LOG_WARNING("setpriority() failed");
+        LOG_DEBUG("setpriority() failed: %s (%d)", std::strerror(errno), errno);
     }
 
     // Ignore the SIGPIPE signal so that any send to a broken socket will return an error code instead of asserting a signal
     // Handling the error at the send function call is much easier than trying to do anything intelligent in the sig handler
-    signal(SIGPIPE, SIG_IGN);
+    if (signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
+        LOG_ERROR("Error ignoring SIGPIPE");
+    }
 
     // only enable when running in system-wide mode
     bool enable_annotation_listener = isCaptureOperationModeSystemWide(result.mCaptureOperationMode);
