@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2020 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2019-2024 by Arm Limited. All rights reserved. */
 
 #ifndef INCLUDE_LINUX_CORE_ONLINER_H
 #define INCLUDE_LINUX_CORE_ONLINER_H
@@ -7,17 +7,21 @@
  * Reads the online state of a cpu, then attempts to bring it online.
  * The destructor will restore the previous state if it was modified
  */
+#include <optional>
 class CoreOnliner {
 public:
     CoreOnliner(unsigned core);
     CoreOnliner(const CoreOnliner &) = delete;
-    CoreOnliner(CoreOnliner &&) noexcept;
+    CoreOnliner(CoreOnliner && that) noexcept;
     ~CoreOnliner();
     CoreOnliner & operator=(const CoreOnliner &) = delete;
-    CoreOnliner & operator=(CoreOnliner &&) noexcept;
-    inline bool stateKnown() const { return known; }
-    inline bool stateChanged() const { return changed; }
-    inline bool isOnline() const { return online; }
+    CoreOnliner & operator=(CoreOnliner && that) noexcept;
+
+    [[nodiscard]] static std::optional<bool> isCoreOnline(unsigned core);
+
+    [[nodiscard]] bool stateKnown() const { return known; }
+    [[nodiscard]] bool stateChanged() const { return changed; }
+    [[nodiscard]] bool isOnline() const { return online; }
 
 private:
     unsigned core;
