@@ -16,6 +16,8 @@
 #include "lib/Span.h"
 #include "lib/Syscall.h"
 #include "lib/Utils.h"
+#include "lib/midr.h"
+#include "linux/CoreOnliner.h"
 #include "linux/smmu_identifier.h"
 #include "linux/smmu_support.h"
 #include "xml/PmuXML.h"
@@ -35,6 +37,7 @@
 #include <string>
 #include <string_view>
 #include <unordered_map>
+#include <utility>
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -139,11 +142,6 @@ namespace {
         return strncmp(string, prefix, strlen(prefix)) == 0;
     }
 
-    [[nodiscard]] bool isValidCpuId(int cpuId)
-    {
-        return ((cpuId != PerfDriverConfiguration::UNKNOWN_CPUID) && (cpuId != -1) && (cpuId != 0));
-    }
-
     [[nodiscard]] bool pmu_supports_strobing_patches(lib::FsEntry const & entry)
     {
         auto const format_path = lib::FsEntry::create(entry, "format");
@@ -159,7 +157,7 @@ namespace {
 
     [[nodiscard]] GatorCpu with_max_counters_value(
         GatorCpu const & gatorCpu,
-        std::unordered_map<int, std::size_t> const & max_event_count_by_cpuid)
+        std::unordered_map<cpu_utils::cpuid_t, std::size_t> const & max_event_count_by_cpuid)
     {
         std::size_t value {};
         bool all_invalid = false;
@@ -216,7 +214,10 @@ namespace {
         }
 
         if (static_cast<int>(value) < gatorCpu.getPmncCounters()) {
-            LOG_WARNING("Detected %zu programmable event counters for %s PMU", value, gatorCpu.getCoreName());
+            LOG_WARNING("Detected %zu programmable event counters for %s PMU (expected %d)",
+                        value,
+                        gatorCpu.getCoreName(),
+                        gatorCpu.getPmncCounters());
         }
 
         LOG_SETUP("CPU Counters\nDetected %zu programmable events for %s", value, gatorCpu.getCoreName());
@@ -224,7 +225,7 @@ namespace {
     }
 
     template<std::size_t max_possible_events>
-    [[nodiscard]] std::size_t counter_valid_readable_counters(int fd)
+    [[nodiscard]] std::size_t count_valid_readable_counters(int fd)
     {
         std::array<std::uint64_t, (2 * max_possible_events) + 1> buffer {};
 
@@ -274,13 +275,14 @@ namespace {
         LOG_DEBUG("Determining max events for #%zu", cpuNo);
 
         // Set the affinity to just that CPU so that it is online, and so that there are some events generated by perf
-        std::unique_ptr<cpu_set_t, std::function<void(cpu_set_t *)>> cpuset {CPU_ALLOC(cpu_set_size),
+        std::unique_ptr<cpu_set_t, std::function<void(cpu_set_t *)>> cpuset {CPU_ALLOC(cpuNo + 1),
                                                                              [](cpu_set_t * ptr) { CPU_FREE(ptr); }};
 
         CPU_ZERO_S(cpu_set_size, cpuset.get());
         CPU_SET_S(cpuNo, cpu_set_size, cpuset.get());
 
-        // try and set affinity
+        // try and set affinity to ensure cpuNo is doing some work (and won't read 0 events)
+        // Note: This needs to be restored elsewhere.
         bool affinitySucceeded = false;
         for (unsigned count = 0; count < affine_loop_count && !affinitySucceeded; ++count) {
             if (sched_setaffinity(0, cpu_set_size, cpuset.get()) == 0) {
@@ -317,7 +319,7 @@ namespace {
             };
 
             if (fd.get() >= 0) {
-                auto const n_valid = counter_valid_readable_counters<max_possible_events>(fd.get());
+                auto const n_valid = count_valid_readable_counters<max_possible_events>(fd.get());
 
                 if (n_valid > event_fds.size()) {
                     event_fds.emplace_back(std::move(fd));
@@ -340,21 +342,22 @@ namespace {
         return actual_count;
     }
 
-    [[nodiscard]] std::unordered_map<int, std::size_t> calculate_max_event_count_by_cpuid(lib::Span<const int> cpuIds)
+    [[nodiscard]] std::unordered_map<cpu_utils::cpuid_t, std::size_t> calculate_max_event_count_by_cpuid(
+        lib::Span<cpu_utils::midr_t const> midrs)
     {
         constexpr unsigned affine_loop_count = 5;
 
-        std::unordered_map<int, std::size_t> max_event_count_by_cpuid {};
+        std::unordered_map<cpu_utils::cpuid_t, std::size_t> max_event_count_by_cpuid {};
 
         if (gSessionData.mOverrideNoPmuSlots > 0) {
             return max_event_count_by_cpuid;
         }
 
         // record the current thread affinity so it can be restored later
-        auto const cpu_set_size = CPU_ALLOC_SIZE(cpuIds.size());
+        auto const cpu_set_size = CPU_ALLOC_SIZE(midrs.size());
 
         std::unique_ptr<cpu_set_t, std::function<void(cpu_set_t *)>> original_cpuset {
-            CPU_ALLOC(cpuIds.size()),
+            CPU_ALLOC(midrs.size()),
             [](cpu_set_t * ptr) { CPU_FREE(ptr); }};
 
         CPU_ZERO_S(cpu_set_size, original_cpuset.get());
@@ -362,26 +365,31 @@ namespace {
         if (sched_getaffinity(0, cpu_set_size, original_cpuset.get()) < 0) {
             auto const e = errno;
             //NOLINTNEXTLINE(concurrency-mt-unsafe)
-            LOG_WARNING("Error calling sched_getaffinity to get current mask: %d (%s)", e, strerror(e));
+            LOG_DEBUG("Error calling sched_getaffinity to get current mask: %d (%s)", e, strerror(e));
 
             CPU_ZERO_S(cpu_set_size, original_cpuset.get());
 
-            for (std::size_t cpuNo = 0; cpuNo < cpuIds.size(); ++cpuNo) {
+            for (std::size_t cpuNo = 0; cpuNo < midrs.size(); ++cpuNo) {
                 CPU_SET_S(cpuNo, cpu_set_size, original_cpuset.get());
             }
         }
 
         // for each CPU, determine how many events can be supported
-        for (std::size_t cpuNo = 0; cpuNo < cpuIds.size(); ++cpuNo) {
+        for (std::size_t cpuNo = 0; cpuNo < midrs.size(); ++cpuNo) {
+            if (!CoreOnliner::isCoreOnline(cpuNo).value_or(true)) {
+                LOG_DEBUG("Skipping max events detection for offline core %zu.", cpuNo);
+                continue;
+            }
+
             auto const actual_count = calculate_max_event_count_for(cpuNo);
-            auto [it, inserted] = max_event_count_by_cpuid.try_emplace(cpuIds[cpuNo], actual_count);
+            auto [it, inserted] = max_event_count_by_cpuid.try_emplace(midrs[cpuNo].to_cpuid(), actual_count);
             if (!inserted) {
                 it->second = std::max(it->second, actual_count);
             }
         }
 
         for (auto const & [cpuId, count] : max_event_count_by_cpuid) {
-            LOG_DEBUG("Determined CPUs with ID 0x%05x can support at most %zu events.", cpuId, count);
+            LOG_DEBUG("Determined CPUs with ID 0x%05x can support at most %zu events.", cpuId.to_raw_value(), count);
         }
 
         // restore affinity
@@ -418,7 +426,7 @@ void logCpuNotFound()
 // NOLINTNEXTLINE(readability-function-cognitive-complexity)
 std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(CaptureOperationMode captureOperationMode,
                                                                          const char * tracefsEventsPath,
-                                                                         lib::Span<const int> cpuIds,
+                                                                         lib::Span<const cpu_utils::midr_t> midrs,
                                                                          const default_identifiers_t & smmu_identifiers,
                                                                          const PmuXML & pmuXml)
 {
@@ -450,9 +458,9 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
     const bool use_64bit_register_set = (sizeof(void *) == 8) || has_64bit_uname;
 
     if (kernelVersion < KERNEL_VERSION(3U, 4U, 0U)) {
-        const char error[] = "Unsupported kernel version\nPlease upgrade to 3.4 or later";
-        LOG_SETUP(error);
-        LOG_ERROR(error);
+        static const char * error_message = "Unsupported kernel version\nPlease upgrade to 3.4 or later";
+        LOG_SETUP("%s", error_message);
+        LOG_ERROR("%s", error_message);
         return nullptr;
     }
 
@@ -476,10 +484,10 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
     int perf_event_paranoid;
     if (lib::readIntFromFile("/proc/sys/kernel/perf_event_paranoid", perf_event_paranoid) != 0) {
         if (isRoot) {
-            const char error[] = "perf_event_paranoid not accessible\n"
-                                 "Is CONFIG_PERF_EVENTS enabled?";
-            LOG_SETUP(error);
-            LOG_ERROR(error);
+            LOG_SETUP("perf_event_paranoid not accessible\n"
+                      "Is CONFIG_PERF_EVENTS enabled?");
+            LOG_ERROR("perf_event_paranoid not accessible\n"
+                      "Is CONFIG_PERF_EVENTS enabled?");
             return nullptr;
         }
 #if defined(CONFIG_ASSUME_PERF_HIGH_PARANOIA) && CONFIG_ASSUME_PERF_HIGH_PARANOIA
@@ -639,6 +647,7 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
         attr.read_format = PERF_FORMAT_ID | PERF_FORMAT_GROUP;
         attr.disabled = 1;
         attr.inherit = 1;
+        attr.inherit_stat = 1; // the original kernel patches require this, the later ones do not (they ignore)
         attr.exclude_kernel = 1;
 
         auto const fd = lib::perf_event_open(&attr, 0, 0, -1, 0);
@@ -656,7 +665,7 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
     }
 
     // detect max number of events per PMU
-    auto const max_event_count_by_cpuid = calculate_max_event_count_by_cpuid(cpuIds);
+    auto const max_event_count_by_cpuid = calculate_max_event_count_by_cpuid(midrs);
 
     // detect the PMUs
     std::set<const GatorCpu *> cpusDetectedViaSysFs;
@@ -678,10 +687,8 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                 const std::string path(lib::Format() << PERF_DEVICES << "/" << name << "/type");
                 if (lib::readIntFromFile(path.c_str(), type) == 0) {
                     LOG_DEBUG("    ... using pmu type %d for %s cores", type, gatorCpu->getCoreName());
-                    configuration->cpus.push_back(PerfCpu {
-                        with_max_counters_value(*gatorCpu, max_event_count_by_cpuid),
-                        type,
-                    });
+                    configuration->cpus.emplace_back(with_max_counters_value(*gatorCpu, max_event_count_by_cpuid),
+                                                     type);
                     cpusDetectedViaSysFs.insert(gatorCpu);
                     if (gatorCpu->getSpeName() != nullptr) {
                         haveFoundKnownCpuWithSpe = true;
@@ -703,7 +710,7 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                                   "system-wide captures.");
                     }
                     else {
-                        configuration->uncores.push_back(PerfUncore {*uncorePmu, type});
+                        configuration->uncores.emplace_back(*uncorePmu, type);
                     }
                     continue;
                 }
@@ -731,34 +738,33 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
 
                     const std::set<int> cpuNumbers = lib::readCpuMaskFromFile(maskPath.c_str());
                     if (!cpuNumbers.empty()) {
-                        int cpuIdForType = 0;
+                        cpu_utils::cpuid_t cpuIdForType;
                         for (int cpuNumber : cpuNumbers) {
                             // track generic pmu's type to the cpuId associated with it.
                             // if multiple different cpuIds are associated, then use -1 as cannot map to a unique pmu.
-                            const int cpuIdForCpu = cpuIds[cpuNumber];
-                            LOG_DEBUG("    ... cpu %d, with cpuid 0x%05x", cpuNumber, cpuIdForCpu);
-                            if (!isValidCpuId(cpuIdForCpu)) {
+                            const auto cpuIdForCpu = midrs[cpuNumber].to_cpuid();
+                            LOG_DEBUG("    ... cpu %d, with cpuid 0x%05x", cpuNumber, cpuIdForCpu.to_raw_value());
+                            if (cpuIdForCpu.invalid_or_other()) {
                                 // skip it as we don't know what it is. fair to assume
                                 // homogeneous clusters.
                                 continue;
                             }
-                            if (cpuIdForType == 0) {
+                            if (!cpuIdForType.valid()) {
                                 cpuIdForType = cpuIdForCpu;
                             }
                             else if (cpuIdForType != cpuIdForCpu) {
-                                cpuIdForType = -1;
+                                cpuIdForType = cpu_utils::cpuid_t::other;
                             }
                         }
-                        if (isValidCpuId(cpuIdForType)) {
+                        if (!cpuIdForType.invalid_or_other()) {
                             const GatorCpu * gatorCpu = pmuXml.findCpuById(cpuIdForType);
                             if (gatorCpu != nullptr) {
                                 LOG_DEBUG("    ... using generic pmu type %d for %s cores",
                                           type,
                                           gatorCpu->getCoreName());
-                                configuration->cpus.push_back(PerfCpu {
+                                configuration->cpus.emplace_back(
                                     with_max_counters_value(*gatorCpu, max_event_count_by_cpuid),
-                                    type,
-                                });
+                                    type);
                                 cpusDetectedViaSysFs.insert(gatorCpu);
                                 if (gatorCpu->getSpeName() != nullptr) {
                                     haveFoundKnownCpuWithSpe = true;
@@ -791,21 +797,20 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
     }
 
     // additionally add any by CPUID
-    std::set<int> unrecognisedCpuIds;
-    for (int cpuId : cpuIds) {
+    std::set<cpu_utils::cpuid_t> unrecognisedCpuIds;
+    for (auto const & midr : midrs) {
+        auto const cpuId = midr.to_cpuid();
         const GatorCpu * gatorCpu = pmuXml.findCpuById(cpuId);
         if (gatorCpu == nullptr) {
             // track the unknown cpuid (filter out some junk values)
-            if (isValidCpuId(cpuId)) {
+            if (!cpuId.invalid_or_other()) {
                 unrecognisedCpuIds.insert(cpuId);
             }
         }
         else if ((cpusDetectedViaSysFs.count(gatorCpu) == 0) && (cpusDetectedViaCpuid.count(gatorCpu) == 0)) {
             LOG_DEBUG("generic pmu: %s", gatorCpu->getCoreName());
-            configuration->cpus.push_back(PerfCpu {
-                with_max_counters_value(*gatorCpu, max_event_count_by_cpuid),
-                PERF_TYPE_RAW,
-            });
+            configuration->cpus.emplace_back(with_max_counters_value(*gatorCpu, max_event_count_by_cpuid),
+                                             PERF_TYPE_RAW);
             cpusDetectedViaCpuid.insert(gatorCpu);
             if (gatorCpu->getSpeName() != nullptr) {
                 haveFoundKnownCpuWithSpe = true;
@@ -841,7 +846,7 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
         for (auto & cpu : configuration->cpus) {
             const auto & currentValue = cpu;
             cpu = PerfCpu {
-                with_max_counters_value(GatorCpu(currentValue.gator_cpu, ARMV82_SPE), max_event_count_by_cpuid),
+                with_max_counters_value(GatorCpu(currentValue.gator_cpu, ARMV82_SPE, "v1p1"), max_event_count_by_cpuid),
                 currentValue.pmu_type,
             };
         }
@@ -849,61 +854,61 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
 
     // insert 'Other' for unknown CPUIDs / no cpus found
     if ((hasNoCpus || addOtherForUnknownSpe) && unrecognisedCpuIds.empty()) {
-        unrecognisedCpuIds.insert(UNKNOWN_CPUID);
+        unrecognisedCpuIds.insert(cpu_utils::cpuid_t::other);
     }
 
     if (!unrecognisedCpuIds.empty()) {
         logCpuNotFound();
         const char * const speName = (addOtherForUnknownSpe ? ARMV82_SPE : nullptr);
+        const char * const speVersion = (addOtherForUnknownSpe ? "v1p1" : nullptr);
+
+        const int defaultPmncCounters = 6;
 
 #if defined(__aarch64__)
-        configuration->cpus.push_back(PerfCpu {
-            with_max_counters_value(
-                GatorCpu {
-                    "Other",
-                    "Other",
-                    "Other",
-                    nullptr,
-                    speName,
-                    unrecognisedCpuIds,
-                    6,
-                    true,
-                },
-                max_event_count_by_cpuid),
-            PERF_TYPE_RAW,
-        });
+        configuration->cpus.emplace_back(with_max_counters_value(
+                                             GatorCpu {
+                                                 "Other",
+                                                 "Other",
+                                                 "Other",
+                                                 nullptr,
+                                                 speName,
+                                                 speVersion,
+                                                 unrecognisedCpuIds,
+                                                 defaultPmncCounters,
+                                                 true,
+                                             },
+                                             max_event_count_by_cpuid),
+                                         PERF_TYPE_RAW);
 #elif defined(__arm__)
-        configuration->cpus.push_back(PerfCpu {
-            with_max_counters_value(
-                GatorCpu {
-                    "Other",
-                    "Other",
-                    "Other",
-                    nullptr,
-                    speName,
-                    unrecognisedCpuIds,
-                    6,
-                    anyV8,
-                },
-                max_event_count_by_cpuid),
-            PERF_TYPE_RAW,
-        });
+        configuration->cpus.emplace_back(with_max_counters_value(
+                                             GatorCpu {
+                                                 "Other",
+                                                 "Other",
+                                                 "Other",
+                                                 nullptr,
+                                                 speName,
+                                                 speVersion,
+                                                 unrecognisedCpuIds,
+                                                 defaultPmncCounters,
+                                                 anyV8,
+                                             },
+                                             max_event_count_by_cpuid),
+                                         PERF_TYPE_RAW);
 #else
-        configuration->cpus.push_back(PerfCpu {
-            with_max_counters_value(
-                GatorCpu {
-                    "Other",
-                    "Perf_Hardware",
-                    "Perf_Hardware",
-                    nullptr,
-                    speName,
-                    unrecognisedCpuIds,
-                    6,
-                    false,
-                },
-                max_event_count_by_cpuid),
-            PERF_TYPE_HARDWARE,
-        });
+        configuration->cpus.emplace_back(with_max_counters_value(
+                                             GatorCpu {
+                                                 "Other",
+                                                 "Perf_Hardware",
+                                                 "Perf_Hardware",
+                                                 nullptr,
+                                                 speName,
+                                                 speVersion,
+                                                 unrecognisedCpuIds,
+                                                 defaultPmncCounters,
+                                                 false,
+                                             },
+                                             max_event_count_by_cpuid),
+                                         PERF_TYPE_HARDWARE);
 #endif
     }
 
