@@ -1,10 +1,11 @@
-/* Copyright (C) 2010-2023 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2010-2024 by Arm Limited. All rights reserved. */
 
 #include "xml/PmuXMLParser.h"
 
 #include "Logging.h"
 #include "OlyUtility.h"
 #include "lib/FsEntry.h"
+#include "lib/midr.h"
 #include "linux/smmu_identifier.h"
 #include "xml/PmuXML.h"
 
@@ -39,6 +40,7 @@ namespace {
     constexpr auto ATTR_CORE_NAME = "core_name"sv;
     constexpr auto ATTR_DT_NAME = "dt_name"sv;
     constexpr auto ATTR_SPE_NAME = "spe"sv;
+    constexpr auto ATTR_SPE_VERSION = "spe_version"sv;
     constexpr auto ATTR_PMNC_COUNTERS = "pmnc_counters"sv;
     constexpr auto ATTR_PROFILE = "profile"sv;
     constexpr auto ATTR_HAS_CYCLES_COUNTER = "has_cycles_counter"sv;
@@ -51,8 +53,6 @@ namespace {
     constexpr auto SMMUV3_DEFAULT_TBU_COUNTER_SET = "SMMUv3_TBU"sv;
     constexpr auto SMMUV3_DEFAULT_TCU_COUNTER_SET = "SMMUv3_TCU"sv;
 
-    constexpr unsigned int cpuid_mask = 0xfffff;
-
     constexpr auto perf_devices = "/sys/bus/event_source/devices"sv;
 
     // NOLINTNEXTLINE(bugprone-easily-swappable-parameters)
@@ -115,7 +115,7 @@ namespace {
         return strcasecmp(pmu_name, test_name) == 0;
     }
 
-    bool parseCpuId(std::set<int> & cpuIds,
+    bool parseCpuId(std::set<cpu_utils::cpuid_t> & cpuIds,
                     const char * const cpuIdStr,
                     bool required,
                     const char * const pmuId,
@@ -129,13 +129,15 @@ namespace {
             return true;
         }
 
-        int cpuid;
-        if (!stringToInt(&cpuid, cpuIdStr, 0)) {
+        int raw_cpuid;
+        if (!stringToInt(&raw_cpuid, cpuIdStr, 0)) {
             LOG_ERROR("The %s for '%s' in pmu XML is not an integer", locationStr, pmuId);
             return false;
         }
 
-        if ((cpuid != 0) && ((static_cast<unsigned int>(cpuid) & cpuid_mask) != cpuid_mask)) {
+        auto cpuid = cpu_utils::cpuid_t ::from_raw(raw_cpuid);
+
+        if (!cpuid.invalid_or_other()) {
             cpuIds.insert(cpuid);
             return true;
         }
@@ -246,11 +248,12 @@ bool parseXml(const char * const xml, PmuXML & pmuXml)
         const char * const coreName = mxmlElementGetAttr(node, ATTR_CORE_NAME.data());
         const char * const dtName = mxmlElementGetAttr(node, ATTR_DT_NAME.data());
         const char * speName = mxmlElementGetAttr(node, ATTR_SPE_NAME.data());
+        const char * speVersion = mxmlElementGetAttr(node, ATTR_SPE_VERSION.data());
         const char * const pmncCountersStr = mxmlElementGetAttr(node, ATTR_PMNC_COUNTERS.data());
         const char * const profileStr = mxmlElementGetAttr(node, ATTR_PROFILE.data());
 
         // read cpuid(s)
-        std::set<int> cpuIds;
+        std::set<cpu_utils::cpuid_t> cpuIds;
         {
             if (!parseCpuId(cpuIds, mxmlElementGetAttr(node, ATTR_CPUID.data()), false, id, "cpuid attribute")) {
                 return false;
@@ -302,7 +305,7 @@ bool parseXml(const char * const xml, PmuXML & pmuXml)
                   ATTR_COUNTER_SET.data(),
                   counterSet,
                   ATTR_CPUID.data(),
-                  *cpuIds.begin(),
+                  cpuIds.begin()->to_raw_value(),
                   ATTR_PMNC_COUNTERS.data(),
                   pmncCounters);
 
@@ -321,10 +324,12 @@ bool parseXml(const char * const xml, PmuXML & pmuXml)
 
             if (!speDeviceFound) {
                 speName = nullptr;
+                speVersion = nullptr;
             }
         }
 
-        pmuXml.cpus.emplace_back(coreName, id, counterSet, dtName, speName, std::move(cpuIds), pmncCounters, isV8);
+        pmuXml.cpus
+            .emplace_back(coreName, id, counterSet, dtName, speName, speVersion, std::move(cpuIds), pmncCounters, isV8);
     }
 
     for (mxml_node_t * node = mxmlFindElement(root, root, TAG_UNCORE_PMU.data(), nullptr, nullptr, MXML_DESCEND);
